@@ -1,10 +1,13 @@
+from __future__ import annotations
+
 import json
 from pathlib import Path
-from typing import Literal, Self
+from typing import Self
 
 from pydantic import BaseModel, Field, SecretStr, ValidationError, field_serializer, model_validator
 
 from kimi_cli.exception import ConfigError
+from kimi_cli.llm import ModelCapability, ProviderType
 from kimi_cli.share import get_share_dir
 from kimi_cli.utils.logging import logger
 
@@ -12,7 +15,7 @@ from kimi_cli.utils.logging import logger
 class LLMProvider(BaseModel):
     """LLM provider configuration."""
 
-    type: Literal["kimi", "openai_legacy", "openai_responses", "anthropic", "_chaos"]
+    type: ProviderType
     """Provider type"""
     base_url: str
     """API base URL"""
@@ -26,9 +29,6 @@ class LLMProvider(BaseModel):
         return v.get_secret_value()
 
 
-LLMModelCapability = Literal["image_in"]
-
-
 class LLMModel(BaseModel):
     """LLM model configuration."""
 
@@ -38,7 +38,7 @@ class LLMModel(BaseModel):
     """Model name"""
     max_context_size: int
     """Maximum context size (unit: tokens)"""
-    capabilities: set[LLMModelCapability] | None = None
+    capabilities: set[ModelCapability] | None = None
     """Model capabilities"""
 
 
@@ -66,11 +66,28 @@ class MoonshotSearchConfig(BaseModel):
         return v.get_secret_value()
 
 
+class MoonshotFetchConfig(BaseModel):
+    """Moonshot Fetch configuration."""
+
+    base_url: str
+    """Base URL for Moonshot Fetch service."""
+    api_key: SecretStr
+    """API key for Moonshot Fetch service."""
+    custom_headers: dict[str, str] | None = None
+    """Custom headers to include in API requests."""
+
+    @field_serializer("api_key", when_used="json")
+    def dump_secret(self, v: SecretStr):
+        return v.get_secret_value()
+
+
 class Services(BaseModel):
     """Services configuration."""
 
     moonshot_search: MoonshotSearchConfig | None = None
     """Moonshot Search configuration."""
+    moonshot_fetch: MoonshotFetchConfig | None = None
+    """Moonshot Fetch configuration."""
 
 
 class Config(BaseModel):
