@@ -1,15 +1,19 @@
+from __future__ import annotations
+
 import asyncio
 import contextlib
 from collections.abc import Callable, Coroutine
 from contextvars import ContextVar
-from typing import Any, NamedTuple, Protocol, runtime_checkable
+from dataclasses import dataclass
+from typing import TYPE_CHECKING, Any, Protocol, runtime_checkable
 
-from kosong.base.message import ContentPart
+from kosong.message import ContentPart
 
-from kimi_cli.llm import LLM
 from kimi_cli.utils.logging import logger
-from kimi_cli.wire import Wire, WireUISide
-from kimi_cli.wire.message import WireMessage
+from kimi_cli.wire import Wire, WireMessage, WireUISide
+
+if TYPE_CHECKING:
+    from kimi_cli.llm import LLM, ModelCapability
 
 
 class LLMNotSet(Exception):
@@ -21,12 +25,12 @@ class LLMNotSet(Exception):
 class LLMNotSupported(Exception):
     """Raised when the LLM does not have required capabilities."""
 
-    def __init__(self, llm: LLM, capabilities: list[str]):
+    def __init__(self, llm: LLM, capabilities: list[ModelCapability]):
         self.llm = llm
         self.capabilities = capabilities
         capabilities_str = "capability" if len(capabilities) == 1 else "capabilities"
         super().__init__(
-            f"The LLM model '{llm.model_name}' does not support required {capabilities_str}: "
+            f"LLM model '{llm.model_name}' does not support required {capabilities_str}: "
             f"{', '.join(capabilities)}."
         )
 
@@ -41,7 +45,8 @@ class MaxStepsReached(Exception):
         self.n_steps = n_steps
 
 
-class StatusSnapshot(NamedTuple):
+@dataclass(frozen=True, slots=True)
+class StatusSnapshot:
     context_usage: float
     """The usage of the context, in percentage."""
 
@@ -54,8 +59,13 @@ class Soul(Protocol):
         ...
 
     @property
-    def model(self) -> str:
-        """The LLM model used by the soul. Empty string indicates no LLM configured."""
+    def model_name(self) -> str:
+        """The name of the LLM model used by the soul. Empty string indicates no LLM configured."""
+        ...
+
+    @property
+    def model_capabilities(self) -> set[ModelCapability] | None:
+        """The capabilities of the LLM model used by the soul. None indicates no LLM configured."""
         ...
 
     @property
@@ -89,7 +99,7 @@ class RunCancelled(Exception):
 
 
 async def run_soul(
-    soul: "Soul",
+    soul: Soul,
     user_input: str | list[ContentPart],
     ui_loop_fn: UILoopFn,
     cancel_event: asyncio.Event,
@@ -147,8 +157,8 @@ async def run_soul(
             pass
         except TimeoutError:
             logger.warning("UI loop timed out")
-
-        _current_wire.reset(wire_token)
+        finally:
+            _current_wire.reset(wire_token)
 
 
 _current_wire = ContextVar[Wire | None]("current_wire", default=None)
