@@ -1,14 +1,14 @@
 from pathlib import Path
 from typing import Any, override
 
-import aiofiles
 from kosong.tooling import CallableTool2, ToolError, ToolOk, ToolReturnType
 from pydantic import BaseModel, Field
 
+from kaos.path import KaosPath
+from kimi_cli.soul.agent import BuiltinSystemPromptArgs
 from kimi_cli.soul.approval import Approval
-from kimi_cli.soul.runtime import BuiltinSystemPromptArgs
 from kimi_cli.tools.file import FileActions
-from kimi_cli.tools.utils import ToolRejectedError
+from kimi_cli.tools.utils import ToolRejectedError, load_desc
 
 
 class Edit(BaseModel):
@@ -29,7 +29,7 @@ class Params(BaseModel):
 
 class StrReplaceFile(CallableTool2[Params]):
     name: str = "StrReplaceFile"
-    description: str = (Path(__file__).parent / "replace.md").read_text(encoding="utf-8")
+    description: str = load_desc(Path(__file__).parent / "replace.md")
     params: type[Params] = Params
 
     def __init__(self, builtin_args: BuiltinSystemPromptArgs, approval: Approval, **kwargs: Any):
@@ -37,14 +37,13 @@ class StrReplaceFile(CallableTool2[Params]):
         self._work_dir = builtin_args.KIMI_WORK_DIR
         self._approval = approval
 
-    def _validate_path(self, path: Path) -> ToolError | None:
+    async def _validate_path(self, path: KaosPath) -> ToolError | None:
         """Validate that the path is safe to edit."""
         # Check for path traversal attempts
-        resolved_path = path.resolve()
-        resolved_work_dir = self._work_dir.resolve()
+        resolved_path = path.canonical()
 
         # Ensure the path is within work directory
-        if not str(resolved_path).startswith(str(resolved_work_dir)):
+        if not str(resolved_path).startswith(str(self._work_dir)):
             return ToolError(
                 message=(
                     f"`{path}` is outside the working directory. "
@@ -64,7 +63,7 @@ class StrReplaceFile(CallableTool2[Params]):
     @override
     async def __call__(self, params: Params) -> ToolReturnType:
         try:
-            p = Path(params.path)
+            p = KaosPath(params.path)
 
             if not p.is_absolute():
                 return ToolError(
@@ -76,16 +75,16 @@ class StrReplaceFile(CallableTool2[Params]):
                 )
 
             # Validate path safety
-            path_error = self._validate_path(p)
+            path_error = await self._validate_path(p)
             if path_error:
                 return path_error
 
-            if not p.exists():
+            if not await p.exists():
                 return ToolError(
                     message=f"`{params.path}` does not exist.",
                     brief="File not found",
                 )
-            if not p.is_file():
+            if not await p.is_file():
                 return ToolError(
                     message=f"`{params.path}` is not a file.",
                     brief="Invalid path",
@@ -100,8 +99,7 @@ class StrReplaceFile(CallableTool2[Params]):
                 return ToolRejectedError()
 
             # Read the file content
-            async with aiofiles.open(p, encoding="utf-8", errors="replace") as f:
-                content = await f.read()
+            content = await p.read_text(errors="replace")
 
             original_content = content
             edits = [params.edit] if isinstance(params.edit, Edit) else params.edit
@@ -118,8 +116,7 @@ class StrReplaceFile(CallableTool2[Params]):
                 )
 
             # Write the modified content back to the file
-            async with aiofiles.open(p, mode="w", encoding="utf-8") as f:
-                await f.write(content)
+            await p.write_text(content, errors="replace")
 
             # Count changes for success message
             total_replacements = 0
