@@ -1,14 +1,14 @@
 from pathlib import Path
 from typing import Any, Literal, override
 
-import aiofiles
 from kosong.tooling import CallableTool2, ToolError, ToolOk, ToolReturnType
 from pydantic import BaseModel, Field
 
+from kaos.path import KaosPath
+from kimi_cli.soul.agent import BuiltinSystemPromptArgs
 from kimi_cli.soul.approval import Approval
-from kimi_cli.soul.runtime import BuiltinSystemPromptArgs
 from kimi_cli.tools.file import FileActions
-from kimi_cli.tools.utils import ToolRejectedError
+from kimi_cli.tools.utils import ToolRejectedError, load_desc
 
 
 class Params(BaseModel):
@@ -26,7 +26,7 @@ class Params(BaseModel):
 
 class WriteFile(CallableTool2[Params]):
     name: str = "WriteFile"
-    description: str = (Path(__file__).parent / "write.md").read_text(encoding="utf-8")
+    description: str = load_desc(Path(__file__).parent / "write.md")
     params: type[Params] = Params
 
     def __init__(self, builtin_args: BuiltinSystemPromptArgs, approval: Approval, **kwargs: Any):
@@ -34,14 +34,13 @@ class WriteFile(CallableTool2[Params]):
         self._work_dir = builtin_args.KIMI_WORK_DIR
         self._approval = approval
 
-    def _validate_path(self, path: Path) -> ToolError | None:
+    async def _validate_path(self, path: KaosPath) -> ToolError | None:
         """Validate that the path is safe to write."""
         # Check for path traversal attempts
-        resolved_path = path.resolve()
-        resolved_work_dir = self._work_dir.resolve()
+        resolved_path = path.canonical()
 
         # Ensure the path is within work directory
-        if not str(resolved_path).startswith(str(resolved_work_dir)):
+        if not str(resolved_path).startswith(str(self._work_dir)):
             return ToolError(
                 message=(
                     f"`{path}` is outside the working directory. "
@@ -57,7 +56,7 @@ class WriteFile(CallableTool2[Params]):
         # - check if the path may contain secrets
         # - check if the file format is writable
         try:
-            p = Path(params.path)
+            p = KaosPath(params.path)
 
             if not p.is_absolute():
                 return ToolError(
@@ -69,11 +68,11 @@ class WriteFile(CallableTool2[Params]):
                 )
 
             # Validate path safety
-            path_error = self._validate_path(p)
+            path_error = await self._validate_path(p)
             if path_error:
                 return path_error
 
-            if not p.parent.exists():
+            if not await p.parent.exists():
                 return ToolError(
                     message=f"`{params.path}` parent directory does not exist.",
                     brief="Parent directory not found",
@@ -97,15 +96,15 @@ class WriteFile(CallableTool2[Params]):
             ):
                 return ToolRejectedError()
 
-            # Determine file mode for aiofiles
-            file_mode = "w" if params.mode == "overwrite" else "a"
-
             # Write content to file
-            async with aiofiles.open(p, mode=file_mode, encoding="utf-8") as f:
-                await f.write(params.content)
+            match params.mode:
+                case "overwrite":
+                    await p.write_text(params.content)
+                case "append":
+                    await p.append_text(params.content)
 
             # Get file info for success message
-            file_size = p.stat().st_size
+            file_size = (await p.stat()).st_size
             action = "overwritten" if params.mode == "overwrite" else "appended to"
             return ToolOk(
                 output="",
