@@ -1,21 +1,21 @@
+from __future__ import annotations
+
 import asyncio
 import json
 import sys
 from functools import partial
 from pathlib import Path
 
-import aiofiles
-from kosong.base.message import Message
 from kosong.chat_provider import ChatProviderError
+from kosong.message import Message
 from rich import print
 
 from kimi_cli.cli import InputFormat, OutputFormat
 from kimi_cli.soul import LLMNotSet, MaxStepsReached, RunCancelled, Soul, run_soul
+from kimi_cli.ui.print.visualize import visualize
 from kimi_cli.utils.logging import logger
 from kimi_cli.utils.message import message_extract_text
 from kimi_cli.utils.signals import install_sigint_handler
-from kimi_cli.wire import WireUISide
-from kimi_cli.wire.message import StepInterrupted
 
 
 class PrintApp:
@@ -37,8 +37,8 @@ class PrintApp:
         context_file: Path,
     ):
         self.soul = soul
-        self.input_format = input_format
-        self.output_format = output_format
+        self.input_format: InputFormat = input_format
+        self.output_format: OutputFormat = output_format
         self.context_file = context_file
 
     async def run(self, command: str | None = None) -> bool:
@@ -69,12 +69,13 @@ class PrintApp:
                 if command:
                     logger.info("Running agent with command: {command}", command=command)
                     if self.output_format == "text":
-                        visualize_fn = self._visualize_text
                         print(command)
-                    else:
-                        assert self.output_format == "stream-json"
-                        visualize_fn = partial(self._visualize_stream_json, start_position=0)
-                    await run_soul(self.soul, command, visualize_fn, cancel_event)
+                    await run_soul(
+                        self.soul,
+                        command,
+                        partial(visualize, self.output_format),
+                        cancel_event,
+                    )
                 else:
                     logger.info("Empty command, skipping")
 
@@ -123,31 +124,3 @@ class PrintApp:
                 )
             except Exception:
                 logger.warning("Ignoring invalid user message: {json_line}", json_line=json_line)
-
-    async def _visualize_text(self, wire: WireUISide):
-        while True:
-            msg = await wire.receive()
-            print(msg)
-            if isinstance(msg, StepInterrupted):
-                break
-
-    async def _visualize_stream_json(self, wire: WireUISide, start_position: int):
-        # TODO: be aware of context compaction
-        # FIXME: this is only a temporary impl, may miss the last lines of the context file
-        if not self.context_file.exists():
-            self.context_file.touch()
-        async with aiofiles.open(self.context_file, encoding="utf-8") as f:
-            await f.seek(start_position)
-            while True:
-                should_end = False
-                while (msg := wire.receive_nowait()) is not None:
-                    if isinstance(msg, StepInterrupted):
-                        should_end = True
-
-                line = await f.readline()
-                if not line:
-                    if should_end:
-                        break
-                    await asyncio.sleep(0.1)
-                    continue
-                print(line, end="")
